from django.utils.formats import localize_input
from django.template.defaultfilters import register

def do_linebreaks(value):
    return value.replace('\n', '\\\\\n')

def do_latex_escape(value):
    return (value
        .replace('&', '\\&')
        .replace('$', '\\$')
        .replace('%', '\\%')
        .replace('#', '\\#')
        .replace('_', '\\_')
        .replace('{', '\\{')
        .replace('}', '\\}')
        )

tex_specific_filters = {
    'localize': localize_input,
    'linebreaks': do_linebreaks,
    'latex_escape': do_latex_escape,
}

FILTERS = register.filters.copy()
FILTERS.update(tex_specific_filters)



from django.db import models
from django.core.exceptions import ValidationError
from django.template import TemplateDoesNotExist
from django.utils.translation import ugettext_lazy as _
from django.template.loader import get_template


def validate_template_path(name):
    try:
        get_template(name, using='tex')
    except TemplateDoesNotExist:
        raise ValidationError(_('Template not found.'))


class TeXTemplateFile(models.Model):
    title = models.CharField(max_length=255)
    name = models.CharField(max_length=255, validators=[validate_template_path, ])

    class Meta:
        abstract = True



import warnings

def render_to_pdf(*args, **kwargs):
    from django_tex.shortcuts import render_to_pdf as func
    warnings.warn(
        ('"django_tex.views.render_to_pdf" is deprecated '
        'and will be removed in future releases. Please use '
        '"django_tex.shortcuts.render_to_pdf" instead.'),
        PendingDeprecationWarning
    ) 
    return func(*args, **kwargs)



from django.conf import settings
from jinja2.ext import Extension
from jinja2 import nodes

def format_path_for_latex(path):
    path = path.replace('\\', '/')
    if not path.endswith('/'):
        path += '/'
    path = '{"' + path + '"}'
    return path

class GraphicspathExtension(Extension):
    """Adds a `graphicspath` tag to Jinja2 that 
    prints out a \graphicspath{ {<path>} } command, where
    <path> is derived from the LATEX_GRAPHICSPATH setting or 
    the BASE_DIR setting by default. 
    """
    tags = set(['graphicspath'])

    def parse(self, parser):
        list_of_paths = getattr(settings, 'LATEX_GRAPHICSPATH', [settings.BASE_DIR])
        value = '\graphicspath{ ' + ' '.join(map(format_path_for_latex, list_of_paths)) + ' }'
        node = nodes.Output(lineno=next(parser.stream).lineno)
        node.nodes = [nodes.MarkSafe(nodes.Const(value))]
        return node



from jinja2 import Environment

from django_tex.filters import FILTERS

def environment(**options):
    options.update(
        {
            'autoescape': None,
            'extensions': ['django_tex.extensions.GraphicspathExtension']
        }
    )
    env = Environment(**options)
    env.filters = FILTERS
    return env



from django.template.backends.jinja2 import Jinja2

DEFAULT_ENVIRONMENT = 'django_tex.environment.environment'

class TeXEngine(Jinja2):
    app_dirname = 'templates'

    def __init__(self, params):
        if 'environment' not in params['OPTIONS'] or not params['OPTIONS']['environment']:
            params['OPTIONS']['environment'] = DEFAULT_ENVIRONMENT
        super().__init__(params)



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name='django-tex',
    description='A simple Django app to render Latex templates and compile them into PDF files.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='https://github.com/weinbusch/django-tex',
    author='Martin Bierbaum',
    license='MIT',
    keywords='django latex jinja2',
    packages=find_packages(exclude=['tests']),
    use_scm_version=True,
    setup_requires=[
        'setuptools_scm',
    ],
    install_requires=[
        'django>=1.11',
        'jinja2>=2.9',
    ],
    python_requires='>=3.6.2',
    package_data={
        '': ['*.tex'],
    },
)



import re


def prettify_message(message):
    '''
    Helper methods that removes consecutive whitespaces and newline characters
    '''
    # Replace consecutive whitespaces with a single whitespace
    message = re.sub(r'[ ]{2,}', ' ', message)
    # Replace consecutive newline characters, optionally separated by whitespace, with a single newline
    message = re.sub(r'([\r\n][ \t]*)+', '\n', message)
    return message


def tokenizer(code):
    token_specification = [
        ('ERROR', r'\! (.+[\r\n])+[\r\n]*'),
        ('WARNING', r'latex warning.*'),
        ('NOFILE', r'no file.*')
    ]
    token_regex = '|'.join('(?P<{}>{})'.format(label, regex) for label, regex in token_specification)
    for m in re.finditer(token_regex, code, re.IGNORECASE):
        token_dict = dict(type=m.lastgroup, message=prettify_message(m.group()))
        yield token_dict


class TexError(Exception):

    def __init__(self, log, source):
        self.log = log
        self.source = source
        self.tokens = list(tokenizer(self.log))
        self.message = self.get_message()

    def get_message(self):
        for token in self.tokens:
            if token['type'] == 'ERROR':
                return token['message']
        return 'No error message found in log'

    def __str__(self):
        return self.message
